@@ -3,11 +3,11 @@
    bitarray is published under the PSF license.
 
    This file is the C part of the bitarray package.
-   All functionality is implemented here.
+   All functionality of bitarray object is implemented here.
 
    Author: Ilan Schnell
 */
-#define BITARRAY_VERSION  "1.5.1"
+#define BITARRAY_VERSION  "1.5.2"
 
 #define PY_SSIZE_T_CLEAN
 #include "Python.h"
@@ -146,7 +146,7 @@ resize(bitarrayobject *self, Py_ssize_t nbits)
     }
 
     if (newsize == size) {
-        /* the memory size hasn't changed - bypass everything */
+        /* the buffer size hasn't changed - bypass everything */
         self->nbits = nbits;
         return 0;
     }
@@ -486,6 +486,9 @@ search(bitarrayobject *self, bitarrayobject *xa, Py_ssize_t p)
     Py_ssize_t i;
 
     assert(p >= 0);
+    if (xa->nbits == 1)         /* faster for sparse bitarrays */
+        return findfirst(self, GETBIT(xa, 0), p, self->nbits);
+
     while (p < self->nbits - xa->nbits + 1) {
         for (i = 0; i < xa->nbits; i++)
             if (GETBIT(self, p + i) != GETBIT(xa, i))
@@ -732,7 +735,7 @@ IntBool_AsInt(PyObject *v)
     long x;
 
     if (PyBool_Check(v))
-        return PyObject_IsTrue(v);
+        return v == Py_True;
 
 #if PY_MAJOR_VERSION == 2
     if (PyInt_Check(v)) {
@@ -960,9 +963,9 @@ PyDoc_STRVAR(buffer_info_doc,
 "buffer_info() -> tuple\n\
 \n\
 Return a tuple (address, size, endianness, unused, allocated) giving the\n\
-memory address of the bitarray's data, the size (in bytes) used to hold the\n\
-bitarray's contents, the bit endianness as a string, the number of unused\n\
-bits in the last bytes, and the size (in bytes) of the allocated memory.");
+memory address of the bitarray's buffer, the buffer size (in bytes),\n\
+the bit endianness as a string, the number of unused bits within the last\n\
+byte, and the allocated memory for the buffer (in bytes).");
 
 
 static PyObject *
@@ -1589,6 +1592,9 @@ bitarray_concat(bitarrayobject *self, PyObject *other)
     PyObject *res;
 
     res = bitarray_copy(self);
+    if (res == NULL)
+        return NULL;
+
     if (extend_dispatch((bitarrayobject *) res, other) < 0) {
         Py_DECREF(res);
         return NULL;
@@ -1602,6 +1608,9 @@ bitarray_repeat(bitarrayobject *self, Py_ssize_t n)
     PyObject *res;
 
     res = bitarray_copy(self);
+    if (res == NULL)
+        return NULL;
+
     if (repeat((bitarrayobject *) res, n) < 0) {
         Py_DECREF(res);
         return NULL;
@@ -1873,6 +1882,9 @@ bitarray_cpinvert(bitarrayobject *self)
     PyObject *result;
 
     result = bitarray_copy(self);
+    if (result == NULL)
+        return NULL;
+
     invert((bitarrayobject *) result);
     return result;
 }
@@ -1930,6 +1942,8 @@ bitarray_ ## oper (bitarrayobject *self, PyObject *other)           \
     PyObject *res;                                                  \
                                                                     \
     res = bitarray_copy(self);                                      \
+    if (res == NULL)                                                \
+        return NULL;                                                \
     if (bitwise((bitarrayobject *) res, other, OP_ ## oper) < 0) {  \
         Py_DECREF(res);                                             \
         return NULL;                                                \
@@ -2968,7 +2982,7 @@ arbitrary.  If you want all values to be set, use the .setall() method.\n\
 element in the sequence is converted to a bit using its truth value.\n\
 \n\
 `bitarray`: Create bitarray from another bitarray.  This is done by\n\
-copying the memory holding the bitarray data, and is hence very fast.\n\
+copying the buffer holding the bitarray data, and is hence very fast.\n\
 \n\
 The optional keyword arguments `endian` specifies the bit endianness of the\n\
 created bitarray object.\n\
@@ -3177,6 +3191,8 @@ init_bitarray(void)
     m = PyModule_Create(&moduledef);
     if (m == NULL)
         return NULL;
+    if (PyType_Ready(&Bitarraytype) < 0)
+        return NULL;
 #else
     m = Py_InitModule3("_bitarray", module_functions, 0);
     if (m == NULL)
