@@ -13,7 +13,8 @@ try:
 except ImportError:
     pass
 
-from bitarray import bitarray, frozenbitarray, bits2bytes, _set_default_endian
+from bitarray import (bitarray, frozenbitarray, bits2bytes,
+                      get_default_endian, _set_default_endian)
 from bitarray.test_bitarray import Util
 
 from bitarray.util import (zeros, make_endian, rindex, strip, count_n,
@@ -108,9 +109,9 @@ tests.append(TestsMakeEndian)
 
 # ---------------------------------------------------------------------------
 
-class TestsHelpers(unittest.TestCase, Util):
+class TestsRindex(unittest.TestCase, Util):
 
-    def test_rindex(self):
+    def test_simple(self):
         self.assertRaises(TypeError, rindex)
         self.assertRaises(TypeError, rindex, None)
         self.assertRaises(TypeError, rindex, bitarray(), 1, 2)
@@ -139,7 +140,7 @@ class TestsHelpers(unittest.TestCase, Util):
             self.assertRaises(ValueError, rindex,
                               bitarray('11111', endian), 0)
 
-    def test_rindex2(self):
+    def test_random(self):
         for a in self.randombitarrays():
             v = randint(0, 1)
             try:
@@ -153,7 +154,7 @@ class TestsHelpers(unittest.TestCase, Util):
                 j = None
             self.assertEqual(i, j)
 
-    def test_rindex3(self):
+    def test_3(self):
         for _ in range(100):
             n = randint(1, 100000)
             v = randint(0, 1)
@@ -164,7 +165,7 @@ class TestsHelpers(unittest.TestCase, Util):
                 a[i] = v
             self.assertEqual(rindex(a, v), max(lst))
 
-    def test_rindex4(self):
+    def test_one_set(self):
         for _ in range(100):
             N = randint(1, 10000)
             a = bitarray(N)
@@ -172,7 +173,13 @@ class TestsHelpers(unittest.TestCase, Util):
             a[randint(0, N - 1)] = 1
             self.assertEqual(rindex(a), a.index(1))
 
-    def test_strip1(self):
+tests.append(TestsRindex)
+
+# ---------------------------------------------------------------------------
+
+class TestsStrip(unittest.TestCase, Util):
+
+    def test_simple(self):
         self.assertRaises(TypeError, strip, '0110')
         self.assertRaises(TypeError, strip, bitarray(), 123)
         self.assertRaises(ValueError, strip, bitarray(), 'up')
@@ -189,7 +196,7 @@ class TestsHelpers(unittest.TestCase, Util):
             self.assertEqual(strip(bitarray('000'), mode), bitarray())
             self.assertEqual(strip(bitarray(), mode), bitarray())
 
-    def test_strip2(self):
+    def test_random(self):
         for a in self.randombitarrays():
             b = a.copy()
             s = a.to01()
@@ -198,7 +205,7 @@ class TestsHelpers(unittest.TestCase, Util):
             self.assertEqual(strip(a, 'both'), bitarray(s.strip('0')))
             self.assertEQUAL(a, b)
 
-    def test_strip_both(self):
+    def test_one_set(self):
         for _ in range(100):
             N = randint(1, 10000)
             a = bitarray(N)
@@ -206,12 +213,18 @@ class TestsHelpers(unittest.TestCase, Util):
             a[randint(0, N - 1)] = 1
             self.assertEqual(strip(a, 'both'), bitarray('1'))
 
+tests.append(TestsStrip)
+
+# ---------------------------------------------------------------------------
+
+class TestsCount_N(unittest.TestCase, Util):
+
     def check_result(self, a, n, i):
         self.assertEqual(a.count(1, 0, i), n)
         if i:
             self.assertTrue(a[i - 1])
 
-    def test_count_n1(self):
+    def test_simple(self):
         a = bitarray('111110111110111110111110011110111110111110111000')
         b = a.copy()
         self.assertEqual(len(a), 48)
@@ -220,8 +233,7 @@ class TestsHelpers(unittest.TestCase, Util):
         self.assertEqual(count_n(a, 0), 0)
         self.assertEqual(count_n(a, 20), 23)
         self.assertEqual(count_n(a, 37), 45)
-        self.assertRaisesMessage(ValueError,
-                                 "non-negative integer expected",
+        self.assertRaisesMessage(ValueError, "non-negative integer expected",
                                  count_n, a, -1) # n < 0
         self.assertRaisesMessage(ValueError, "n larger than bitarray size",
                                  count_n, a, 49) # n > len(a)
@@ -234,7 +246,7 @@ class TestsHelpers(unittest.TestCase, Util):
             self.assertEqual(a[:i].count(), n)
         self.assertEQUAL(a, b)
 
-    def test_count_n1_frozen(self):
+    def test_frozen(self):
         a = frozenbitarray('001111101111101111101111100111100')
         self.assertEqual(len(a), 33)
         self.assertEqual(a.count(), 24)
@@ -247,7 +259,7 @@ class TestsHelpers(unittest.TestCase, Util):
         self.assertRaises(ValueError, count_n, a, 34) # n > len(a)
         self.assertRaises(TypeError, count_n, a, "7")
 
-    def test_count_n2(self):
+    def test_large(self):
         for N in list(range(100)) + [1000, 10000, 100000]:
             a = bitarray(N)
             v = randint(0, 1)
@@ -261,7 +273,7 @@ class TestsHelpers(unittest.TestCase, Util):
             self.assertTrue(count_n(a, tc) <= N)
             self.assertRaises(ValueError, count_n, a, tc + 1)
 
-    def test_count_n3(self):
+    def test_one_set(self):
         N = 100000
         for _ in range(10):
             a = bitarray(N)
@@ -273,13 +285,13 @@ class TestsHelpers(unittest.TestCase, Util):
             self.assertEqual(count_n(a, 1), i + 1)
             self.assertRaises(ValueError, count_n, a, 2)
 
-    def test_count_n4(self):
+    def test_random(self):
         for a in self.randombitarrays():
             n = a.count() // 2
             i = count_n(a, n)
             self.check_result(a, n, i)
 
-tests.append(TestsHelpers)
+tests.append(TestsCount_N)
 
 # ---------------------------------------------------------------------------
 
@@ -356,7 +368,7 @@ tests.append(TestsBitwiseCount)
 
 class TestsSubset(unittest.TestCase, Util):
 
-    def test_subset(self):
+    def test_basic(self):
         a = frozenbitarray('0101')
         b = bitarray('0111')
         self.assertTrue(subset(a, b))
@@ -371,7 +383,7 @@ class TestsSubset(unittest.TestCase, Util):
     def subset_simple(self, a, b):
         return (a & b).count() == a.count()
 
-    def test_subset_True(self):
+    def test_True(self):
         for a, b in [('', ''), ('0', '1'), ('0', '0'), ('1', '1'),
                      ('000', '111'), ('0101', '0111'),
                      ('000010111', '010011111')]:
@@ -379,14 +391,14 @@ class TestsSubset(unittest.TestCase, Util):
             self.assertTrue(subset(a, b) is True)
             self.assertTrue(self.subset_simple(a, b) is True)
 
-    def test_subset_False(self):
+    def test_False(self):
         for a, b in [('1', '0'), ('1101', '0111'),
                      ('0000101111', '0100111011')]:
             a, b = bitarray(a), bitarray(b)
             self.assertTrue(subset(a, b) is False)
             self.assertTrue(self.subset_simple(a, b) is False)
 
-    def test_many(self):
+    def test_random(self):
         for a in self.randombitarrays(start=1):
             b = a.copy()
             # we set one random bit in b to 1, so a is always a subset of b
@@ -402,6 +414,20 @@ tests.append(TestsSubset)
 
 # ---------------------------------------------------------------------------
 
+CODEDICT = {'little': {}, 'big': {
+    '0': bitarray('0000'),    '1': bitarray('0001'),
+    '2': bitarray('0010'),    '3': bitarray('0011'),
+    '4': bitarray('0100'),    '5': bitarray('0101'),
+    '6': bitarray('0110'),    '7': bitarray('0111'),
+    '8': bitarray('1000'),    '9': bitarray('1001'),
+    'a': bitarray('1010'),    'b': bitarray('1011'),
+    'c': bitarray('1100'),    'd': bitarray('1101'),
+    'e': bitarray('1110'),    'f': bitarray('1111'),
+}}
+for k, v in CODEDICT['big'].items():
+    CODEDICT['little'][k] = v[::-1]
+
+
 class TestsHexlify(unittest.TestCase, Util):
 
     def test_swap_hilo_bytes(self):
@@ -456,6 +482,16 @@ class TestsHexlify(unittest.TestCase, Util):
         self.assertRaises(Exception, hex2ba, '01a7x89')
         self.assertRaises(TypeError, hex2ba, 0)
 
+    @staticmethod
+    def hex2ba(s, endian=None):
+        a = bitarray(0, endian or get_default_endian())
+        a.encode(CODEDICT[a.endian()], s)
+        return a
+
+    @staticmethod
+    def ba2hex(a):
+        return ''.join(a.iterdecode(CODEDICT[a.endian()]))
+
     def test_explicit(self):
         data = [ #     little  big                  little  big
             ('',       '',     ''),
@@ -467,10 +503,10 @@ class TestsHexlify(unittest.TestCase, Util):
             ('1010',   '5',    'a'),     ('1011',   'd',    'b'),
             ('0110',   '6',    '6'),     ('0111',   'e',    '7'),
             ('1110',   '7',    'e'),     ('1111',   'f',    'f'),
-            ('10001100',         '13',   '8c'),
-            ('100011001110',     '137',  '8ce'),
-            ('1000110011101111', '137f', '8cef'),
-            ('10100111110111001001', '5eb39', 'a7dc9'),
+            ('10001100',             '13',    '8c'),
+            ('100011001110',         '137',   '8ce'),
+            ('1000110011101111',     '137f',  '8cef'),
+            ('10001100111011110100', '137f2', '8cef4'),
         ]
         for bs, hex_le, hex_be in data:
             a_be = bitarray(bs, 'big')
@@ -479,6 +515,11 @@ class TestsHexlify(unittest.TestCase, Util):
             self.assertEQUAL(hex2ba(hex_le, 'little'), a_le)
             self.assertEqual(ba2hex(a_be), hex_be)
             self.assertEqual(ba2hex(a_le), hex_le)
+            # test simple encode / decode implementation
+            self.assertEQUAL(self.hex2ba(hex_be, 'big'), a_be)
+            self.assertEQUAL(self.hex2ba(hex_le, 'little'), a_le)
+            self.assertEqual(self.ba2hex(a_be), hex_be)
+            self.assertEqual(self.ba2hex(a_le), hex_le)
 
     def test_round_trip(self):
         for i in range(100):
@@ -486,14 +527,16 @@ class TestsHexlify(unittest.TestCase, Util):
             for default_endian in 'big', 'little':
                 _set_default_endian(default_endian)
                 a = hex2ba(s)
-                ida = id(a)
                 self.assertEqual(len(a) % 4, 0)
                 self.assertEqual(a.endian(), default_endian)
                 t = ba2hex(a)
                 self.assertEqual(t, s.lower())
                 b = hex2ba(t, default_endian)
                 self.assertEQUAL(a, b)
-                self.assertEqual(id(a), ida)
+                # test simple encode / decode implementation
+                self.assertEQUAL(a, self.hex2ba(t))
+                self.assertEqual(t, self.ba2hex(a))
+
 
 tests.append(TestsHexlify)
 
@@ -523,19 +566,74 @@ class TestsIntegerization(unittest.TestCase, Util):
         self.assertEQUAL(int2ba(6, endian='big'), bitarray('110', 'big'))
         self.assertEQUAL(int2ba(6, endian='little'),
                          bitarray('011', 'little'))
-        self.assertRaises(ValueError, int2ba, -1)
         self.assertRaises(TypeError, int2ba, 1.0)
         self.assertRaises(TypeError, int2ba, 1, 3.0)
         self.assertRaises(ValueError, int2ba, 1, 0)
         self.assertRaises(TypeError, int2ba, 1, 10, 123)
         self.assertRaises(ValueError, int2ba, 1, 10, 'asd')
+        # signed integer requires length
+        self.assertRaises(TypeError, int2ba, 100, signed=True)
+
+    def test_signed(self):
+        for s, i in [
+                ('0',  0),
+                ('1', -1),
+                ('00',  0),
+                ('10',  1),
+                ('01', -2),
+                ('11', -1),
+                ('000',  0),
+                ('100',  1),
+                ('010',  2),
+                ('110',  3),
+                ('001', -4),
+                ('101', -3),
+                ('011', -2),
+                ('111', -1),
+                ('00000',   0),
+                ('11110',  15),
+                ('00001', -16),
+                ('11111',  -1),
+                ('000000000',    0),
+                ('111111110',  255),
+                ('000000001', -256),
+                ('111111111',   -1),
+                ('0000000000000000000000', 0),
+                ('1001000011000000100010', 9 + 3 * 256 + 17 * 2 ** 16),
+                ('1111111111111111111110', 2 ** 21 - 1),
+                ('0000000000000000000001', -2 ** 21),
+                ('1001000011000000100011', -2 ** 21
+                                           + (9 + 3 * 256 + 17 * 2 ** 16)),
+                ('1111111111111111111111', -1),
+        ]:
+            self.assertEqual(ba2int(bitarray(s, 'little'), signed=1), i)
+            self.assertEqual(ba2int(bitarray(s[::-1], 'big'), signed=1), i)
+
+            self.assertEQUAL(int2ba(i, len(s), 'little', signed=1),
+                             bitarray(s, 'little'))
+            self.assertEQUAL(int2ba(i, len(s), 'big', signed=1),
+                             bitarray(s[::-1], 'big'))
+
+    def test_int2ba_overflow(self):
+        self.assertRaises(OverflowError, int2ba, -1)
+        self.assertRaises(OverflowError, int2ba, -1, 4)
+
+        self.assertRaises(OverflowError, int2ba, 128, 7)
+        self.assertRaises(OverflowError, int2ba, 64, 7, signed=1)
+        self.assertRaises(OverflowError, int2ba, -65, 7, signed=1)
+
+        for n in range(1, 20):
+            self.assertRaises(OverflowError, int2ba, 2 ** n, n)
+            self.assertRaises(OverflowError, int2ba, 2 ** (n - 1), n,
+                              signed=1)
+            self.assertRaises(OverflowError, int2ba, -2 ** (n - 1) - 1, n,
+                              signed=1)
 
     def test_int2ba_length(self):
         self.assertRaises(TypeError, int2ba, 0, 1.0)
         self.assertRaises(ValueError, int2ba, 0, 0)
         self.assertEqual(int2ba(5, length=6, endian='big'),
                          bitarray('000101'))
-        self.assertRaises(OverflowError, int2ba, 3, 1)
         for n in range(1, 100):
             ab = int2ba(1, n, 'big')
             al = int2ba(1, n, 'little')
@@ -553,11 +651,12 @@ class TestsIntegerization(unittest.TestCase, Util):
             self.assertEqual(ab, bitarray(n * '0', 'big'))
             self.assertEqual(al, bitarray(n * '0', 'little'))
 
-            self.assertRaises(OverflowError, int2ba, 2 ** n, n, 'big')
-            self.assertRaises(OverflowError, int2ba, 2 ** n, n, 'little')
             self.assertEqual(int2ba(2 ** n - 1), bitarray(n * '1'))
             self.assertEqual(int2ba(2 ** n - 1, endian='little'),
                              bitarray(n * '1'))
+            for endian in 'big', 'little':
+                self.assertEqual(int2ba(-1, n, endian, signed=True),
+                                 bitarray(n * '1'))
 
     def test_explicit(self):
         _set_default_endian('big')
@@ -571,17 +670,6 @@ class TestsIntegerization(unittest.TestCase, Util):
             self.assertEQUAL(int2ba(i, endian='big'), ab)
             self.assertEQUAL(int2ba(i, endian='little'), al)
             self.assertEqual(ba2int(ab), ba2int(al), i)
-            if i == 0 or i >= 512:
-                continue
-            for n in range(9, 32):
-                for endian in 'big', 'little':
-                    a = int2ba(i, length=n, endian=endian)
-                    self.assertEqual(a.endian(), endian)
-                    self.assertEqual(len(a), n)
-                    if endian == 'big':
-                        f = a.index(1)
-                        self.assertEqual(a[:f], bitarray(f * '0'))
-                        self.assertEqual(a[f:], ab)
 
     def check_round_trip(self, i):
         for endian in 'big', 'little':
@@ -594,7 +682,7 @@ class TestsIntegerization(unittest.TestCase, Util):
             self.assertEqual(ba2int(a), i)
             if i > 0 and sys.version_info[:2] >= (2, 7):
                 self.assertEqual(i.bit_length(), len(a))
-            # add a few / trailing leading zeros to bitarray
+            # add a few trailing / leading zeros to bitarray
             if endian == 'big':
                 a = zeros(randint(0, 3), endian) + a
             else:
@@ -607,6 +695,24 @@ class TestsIntegerization(unittest.TestCase, Util):
             self.check_round_trip(i)
             self.check_round_trip(randint(0, 10 ** randint(3, 300)))
 
+    @staticmethod
+    def twos_complement(i, num_bits):
+        # https://en.wikipedia.org/wiki/Two%27s_complement
+        mask = 2 ** (num_bits - 1)
+        return -(i & mask) + (i & ~mask)
+
+    def test_random_signed(self):
+        for a in self.randombitarrays(start=1):
+            i = ba2int(a, signed=True)
+            b = int2ba(i, len(a), a.endian(), signed=True)
+            self.assertEQUAL(a, b)
+
+            j = ba2int(a, signed=False)  # unsigned
+            if i >= 0:
+                self.assertEqual(i, j)
+
+            self.assertEqual(i, self.twos_complement(j, len(a)))
+
 
 tests.append(TestsIntegerization)
 
